#!/usr/bin/perl

# ============================== SUMMARY =====================================
# Program : dell_oob_server_config_generator.pl
# Version : 0.1
# Date    : August 25 2013
# Author  : Vishwanath Patil  - vishwanath_s_patil@dell.com
#
# Summary : This is a nagios plugin that generates host and service definations 
#	    for dell iDRAC7 devices 
#
# Licence : GPL - summary below, full text at http://www.fsf.org/licenses/gpl.txt
#================================================================================

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


use lib "/usr/lib/perl5/vendor_perl/5.8.8/Net";

use strict;
use warnings;
use Net::SNMP;
use Socket;
my $totalArgs = $#ARGV + 1;
if (($totalArgs < 2 ) || ( "$ARGV[0]" ne "-a"))
{
	print ("Invalid args\n");
 	print("Usage: perl dell_oob_server_config_generator.pl -a <IP Address> -c <SNMP community string> -p <SNMP port>\n");
	exit 1; 
}
 
my $deviceIPAddress = $ARGV[1];
my $communityString = "public";
my $snmpPort =161;

if ( ($totalArgs > 2) && ("$ARGV[2]" eq "-c") )
{
	$communityString = $ARGV[3];
}elsif ( ($totalArgs > 3 ) && ( "$ARGV[4]" eq "-c" ) )
{
	$communityString = $ARGV[5];
}

if ( ($totalArgs > 2) && ("$ARGV[2]" eq "-p") )
{
         $snmpPort  = $ARGV[3];
}elsif ( ($totalArgs > 3 ) && ( "$ARGV[4]" eq "-p" ) )
{
          $snmpPort = $ARGV[5];
}

use constant iDRAC7MONOLITHIC 	=> 16;
use constant iDRAC7MODULAR 	=> 17;

#MIB oids to be queried
my $sysNameOid = "1.3.6.1.2.1.1.5.0";
my $deviceTypeOid = '1.3.6.1.4.1.674.10892.5.1.1.7.0';
my $systemOSNameOid = '1.3.6.1.4.1.674.10892.5.1.3.6.0';
my $deviceGlobalHealthStatusOid = '1.3.6.1.4.1.674.10892.5.2.1.0'; 
my $racUrlOid = '1.3.6.1.4.1.674.10892.5.1.1.6.0';
my $fqdnNameOid = '1.3.6.1.4.1.674.10892.5.1.3.1.0';
#####host defination values
my $templateServiceDescription = "Dell Device Health Status";
my $serviceDescription = "Global Health Status";
my $serviceCommandName = "check_dell_device_health";
#my $serviceCommand = "$serviceCommandName! \"$deviceGlobalHealthStatusOid\" \"$communityString\" \"$snmpPort\" ";
my $hostObject = "Dell-OOB-Server";
my $hostGroup = "Dell-OOB-Servers";
my $hostGroupAlias = "Dell OOB Servers";
my $mapImage = "map_dell_idrac7.jpg";
my $image = "dell_idrac7.jpg";
my $perlCommand = "/usr/bin/perl";
#my $commanLine = "<Absolute path of check_dell_device_health.pl file>  \$HOSTADDRESS\$  \$ARG1\$   \$ARG2\$  \$ARG3\$ ";
my $commanLine = "/usr/local/nagios/libexec/check_dell_device_health.pl  \$HOSTADDRESS\$  \$ARG1\$   \$ARG2\$  \$ARG3\$ \$ARG4\$";

###Create SNMP session 
my ($session, $error) = Net::SNMP->session(Hostname => $deviceIPAddress,
                                        Community => $communityString,
                                        port => $snmpPort )   or die "Failed Session: $!\015\012";
if (!defined($session)) {       
  printf("ERROR while creating SNMP session: %s.\n",  $error);     
  exit 1; 
} 

my $deviceType = "";
my $osName = "";
my $aliasName = "DellOOBServer";
my $sysName = "";
my $racURL = "https://$deviceIPAddress/login.html";
my $fqdnName = "<Blank>"; 
my $racTypeResponse = $session->get_request("$deviceTypeOid")  or die "ERROR: Failed to get device type : $!\015\012";
 $deviceType = $racTypeResponse->{"$deviceTypeOid"};

if ($deviceType == iDRAC7MONOLITHIC || $deviceType ==  iDRAC7MODULAR)
{
	my $sysNameMibOidResponse = $session->get_request("$sysNameOid"); 
	$sysName = $sysNameMibOidResponse->{"$sysNameOid"};
	print ("sysName is : $sysName \n");
	my $osNameResponse = $session->get_request("$systemOSNameOid")  ; 
	$osName = $osNameResponse->{"$systemOSNameOid"};
	$osName = lc($osName);
	print "OS is : $osName\n";
	my $racUrlResponse = $session->get_request("$racUrlOid")  ;
	$racURL = $racUrlResponse->{"$racUrlOid"};
	print "RAC URL is : $racURL\n";
	my $fqdNameResponse = $session->get_request("$fqdnNameOid") or die "get_request chassis service tag: $!\015\012";
	$fqdnName = $fqdNameResponse->{"$fqdnNameOid"};
	print "Server host name is: $fqdnName\n";
}
$session->close;

if ("$sysName" eq "")
{
	$sysName = "iDRAC7";
}
$aliasName = $sysName;
if (index( $osName, "linux") != -1) {
    $aliasName = $sysName . "-Linux";
} 

elsif (index( $osName, "windows") != -1) {
	$aliasName = $sysName . "-Windows";    
}
elsif (index( $osName, "esxi") != -1) {
        $aliasName = $sysName . "-ESXi";
}
my $serviceCommand = "$serviceCommandName! \"$deviceGlobalHealthStatusOid\" \"$communityString\" \"$snmpPort\"  \"$fqdnName\" ";

open OUTPUT, '>', "dell_oob_server.cfg" or die "Can't create dell_oob_server.cfg filehandle: $!";
select OUTPUT; $| = 1;  # make unbuffered

print <<CONFIGEND;
#======================================================================================
#               Dell OOB Server host defination
#======================================================================================

define host{
        use                   	$hostObject
        host_name              	$sysName
        alias                  	$aliasName 
        address                	$deviceIPAddress 
	display_name		$sysName
        icon_image      	$image
	hostgroups      	$hostGroup
	statusmap_image   	$mapImage
	action_url		$racURL
}
CONFIGEND

print <<SERVICEEND;
#======================================================================================
# 		Dell OOB Server service defination 
#======================================================================================

define service{
        use                    	$templateServiceDescription
        host_name              	$sysName
        service_description    	$serviceDescription
        check_command          	$serviceCommand
	}

SERVICEEND
print <<HOSTGROUPEND;
#======================================================================================
#               Dell OOB Server host group defination
#======================================================================================

define hostgroup{
        hostgroup_name          $hostGroup
        alias                   $hostGroupAlias
        }

HOSTGROUPEND
close(OUTPUT);


open OUTPUT, '>', "dell_command.cfg" or die "Can't create filehandle: $!";
select OUTPUT; $| = 1;  # make unbuffered

print <<COMMANDEND;
#======================================================================================
#               Dell OOB Server command defination
#======================================================================================

define command{
        command_name	$serviceCommandName 
        command_line    $perlCommand $commanLine 
        }

COMMANDEND

close(OUTPUT);

